@@ -369,7 +369,7 @@ int Monitor(mddev_dev_t devlist,
 				} else if (mse &&  mse->pattern && i < (int)strlen(mse->pattern)) {
 					switch(mse->pattern[i]) {
 					case 'U': newstate = 6 /* ACTIVE/SYNC */; break;
-					case '_': newstate = 0; break;
+					case '_': newstate = 8 /* REMOVED */; break;
 					}
 					disc.major = disc.minor = 0;
 				}
