@@ -376,6 +376,36 @@ int enough(int level, int raid_disks, int layout, int clean,
 	}
 }
 
+int enough_fd(int fd)
+{
+	struct mdu_array_info_s array;
+	struct mdu_disk_info_s disk;
+	int avail_disks = 0;
+	int i;
+	char *avail;
+
+	if (ioctl(fd, GET_ARRAY_INFO, &array) != 0 ||
+	    array.raid_disks <= 0)
+		return 0;
+	avail = calloc(array.raid_disks, 1);
+	for (i=0; i<array.raid_disks + array.nr_disks; i++) {
+		disk.number = i;
+		if (ioctl(fd, GET_DISK_INFO, &disk) != 0)
+			continue;
+		if (! (disk.state & (1<<MD_DISK_SYNC)))
+			continue;
+		if (disk.raid_disk < 0 || disk.raid_disk >= array.raid_disks)
+			continue;
+		avail_disks++;
+		avail[disk.raid_disk] = 1;
+	}
+	/* This is used on an active array, so assume it is clean */
+	return enough(array.level, array.raid_disks, array.layout,
+		      1,
+		      avail, avail_disks);
+}
+
+
 const int uuid_match_any[4] = { ~0, ~0, ~0, ~0 };
 int same_uuid(int a[4], int b[4], int swapuuid)
 {
@@ -959,19 +989,33 @@ int dev_open(char *dev, int flags)
 	int minor;
 
 	if (!dev) return -1;
+	flags |= O_DIRECT;
 
 	major = strtoul(dev, &e, 0);
 	if (e > dev && *e == ':' && e[1] &&
 	    (minor = strtoul(e+1, &e, 0)) >= 0 &&
 	    *e == 0) {
-		snprintf(devname, sizeof(devname), "/dev/.tmp.md.%d:%d:%d",
-			 (int)getpid(), major, minor);
-		if (mknod(devname, S_IFBLK|0600, makedev(major, minor))==0) {
-			fd = open(devname, flags|O_DIRECT);
-			unlink(devname);
+		char *path = map_dev(major, minor, 0);
+		if (path)
+			fd = open(path, flags);
+		if (fd < 0) {
+			snprintf(devname, sizeof(devname), "/dev/.tmp.md.%d:%d:%d",
+				 (int)getpid(), major, minor);
+			if (mknod(devname, S_IFBLK|0600, makedev(major, minor))==0) {
+				fd = open(devname, flags);
+				unlink(devname);
+			}
+		}
+		if (fd < 0) {
+			snprintf(devname, sizeof(devname), "/tmp/.tmp.md.%d:%d:%d",
+				 (int)getpid(), major, minor);
+			if (mknod(devname, S_IFBLK|0600, makedev(major, minor))==0) {
+				fd = open(devname, flags);
+				unlink(devname);
+			}
 		}
 	} else
-		fd = open(dev, flags|O_DIRECT);
+		fd = open(dev, flags);
 	return fd;
 }
 
@@ -980,7 +1024,7 @@ int open_dev(int devnum)
 	char buf[20];
 
 	sprintf(buf, "%d:%d", dev2major(devnum), dev2minor(devnum));
-	return dev_open(buf, O_RDWR);
+	return dev_open(buf, O_RDONLY);
 }
 
 int open_dev_excl(int devnum)
@@ -1427,7 +1471,7 @@ int is_subarray_active(char *subarray, char *container)
 		if (is_container_member(ent, container)) {
 			char *inst = &ent->metadata_version[10+strlen(container)+1];
 
-			if (!subarray || strcmp(inst, subarray) == 0)
+			if (strcmp(inst, subarray) == 0)
 				break;
 		}
 	}
@@ -1437,11 +1481,6 @@ int is_subarray_active(char *subarray, char *container)
 	return ent != NULL;
 }
 
-int is_container_active(char *container)
-{
-	return is_subarray_active(NULL, container);
-}
-
 /* open_subarray - opens a subarray in a container
  * @dev: container device name
  * @st: supertype with only ->subarray set
@@ -1650,7 +1689,7 @@ int stat2devnum(struct stat *st)
 		link[n] = 0;
 		cp = strrchr(link, '/');
 		if (cp) *cp = 0;
-		cp = strchr(link, '/');
+		cp = strrchr(link, '/');
 		if (cp && strncmp(cp, "/md", 3) == 0)
 			return devname2devnum(cp+1);
 	}
